from __future__ import annotations
import abc
import re
import sys

if sys.version_info < (3, 11):
    from typing_extensions import Self
else:
    from typing import Self


class Parsable:
    @staticmethod
    @abc.abstractmethod
    def pattern() -> "re.Pattern[str]": ...

    @classmethod
    def from_str(cls, text: "str") -> "list[Self]":
        return [cls(**m.groupdict()) for m in cls.pattern().finditer(text)]

    @classmethod
    def from_str_or_default(
        cls, text: "str", default: "Self | None" = None
    ) -> "Self | None":
        """return first from the string or some default"""
        res = cls.from_str(text)
        if len(res) == 0:
            return default
        return res[0]



"""module for some regex wrappers for common parsing procedures"""

import sys

if sys.version_info >= (3, 12):
    from parsable.main import Parsable
    from parsable.collection import ParsableCollection
else:
    from parsable.main import Parsable
    from parsable.backports import ParsableCollection

__all__ = ["Parsable", "ParsableCollection"]



from __future__ import annotations
import abc
import sys
from typing import TYPE_CHECKING, TypeVar
from parsable.main import Parsable

if sys.version_info < (3, 11):
    from typing_extensions import Self
else:
    from typing import Self
if TYPE_CHECKING:
    import pandas as pd  # type: ignore

T = TypeVar("T", bound=Parsable)

if sys.version_info < (3, 9):
    to_inherit = list
else:
    to_inherit = list[T]


class ParsableCollection(to_inherit):  # type: ignore[valid-type, name-defined, type-arg]
    @staticmethod
    @abc.abstractmethod
    def runtime_type() -> "type[T]": ...  # type: ignore[name-defined]

    @classmethod
    def from_str(cls, text: "str") -> "Self":
        return cls(cls.runtime_type().from_str(text))  # type: ignore[abstract, attr-defined]

    @property
    def df(self) -> "pd.DataFrame":
        import pandas as pd  # type: ignore

        return pd.DataFrame(self)



from __future__ import annotations
from dataclasses import dataclass
import re
import sys
from parsable import Parsable, ParsableCollection


EXAMPLE_TEXT = """

{
  "border": "{{int(1, 5)}}px {{random(solid, dotted, dashed)}} {{color()}}",
  "coordinates": {
    "type": "array",
    "count": 2,
    "items": "{{float(0, 120, 5)}}"
  },
  "password": "xX{{animal()}}-{{string(6, 10, *)}}"
}

{
  "border": "2px dashed gray",
  "coordinates": [
    14.95685,
    69.91848
  ],
  "password": "xXearthworm-*******"
}

"""


def test_basics() -> None:
    @dataclass
    class Coordinates(Parsable):
        x: float
        y: float

        @staticmethod
        def pattern() -> "re.Pattern[str]":
            return re.compile(
                r"\"coordinates\":\s*\[\s*(?P<x>\d+(?:\.\d+)?)\,\s*(?P<y>\d+(?:\.\d+)?)\,?\s*\]"
            )

        def __post_init__(self) -> None:
            self.x = float(self.x) if isinstance(self.x, str) else self.x
            self.y = float(self.y) if isinstance(self.y, str) else self.y

    coordinates = Coordinates.from_str(EXAMPLE_TEXT)

    assert len(coordinates) == 1
    assert coordinates[0].x == 14.95685
    assert coordinates[0].y == 69.91848

    if sys.version_info < (3, 9):
        to_inherit = ParsableCollection
    else:
        to_inherit = ParsableCollection[Coordinates]

    class CoordinatesCollection(to_inherit):
        @staticmethod
        def runtime_type() -> "type[Coordinates]":
            return Coordinates

    coordinates_alt = CoordinatesCollection.from_str(EXAMPLE_TEXT)
    assert len(coordinates_alt) == 1
    assert coordinates_alt[0].x == 14.95685
    assert coordinates_alt[0].y == 69.91848
